package compbio.controllers;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Calendar;
import java.util.Map;

import org.apache.log4j.Logger;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;

import compbio.statistic.CassandraRequester;
import compbio.beans.ExecutionTimeBean;
import compbio.beans.TotalExecutionTime;
import compbio.cassandra.DateFormatter;
import compbio.cassandra.readers.CassandraReader;
import compbio.cassandra.readers.ExecutionTimeReader;
import compbio.cassandra.readers.JobReader;
import compbio.engine.archive.ArchivedJob;

/**
 * Spring controller for supporting job queries. This version works in the
 * servlet style.
 * 
 * @author Alexander Sherstnev
 * @author Natasha Sherstneva
 * @version 1.0
 * @since Dec 2013
 */
@Controller
public class JobController extends BasicController {
	private static Logger log = Logger.getLogger(JobController.class);

	/**
	 * form a query page for job execution time statistics. The servlet should
	 * be available for users and admins only. By defaults the report time range
	 * is from the earliest day with jobs to today ("Query for all dates" is
	 * ticked). If the user removes the tick the time range is from today - 3
	 * days to today. Currently, the input model is empty.
	 * 
	 * @param model
	 *            MVC model
	 * @return link to the JSP query page
	 */
	@RequestMapping(value = "/stat/exectime/query", method = RequestMethod.GET)
	public String initFormExecTime(Map<String, Object> model) {
		model.put("username", getPrincipalName());
		Calendar cal = Calendar.getInstance();
		String date2 = cal.get(Calendar.YEAR) + "/" + (cal.get(Calendar.MONTH) + 1) + "/" + cal.get(Calendar.DATE);
		cal.add(Calendar.DATE, -3);
		String date1 = cal.get(Calendar.YEAR) + "/" + (cal.get(Calendar.MONTH) + 1) + "/" + cal.get(Calendar.DATE);

		model.put("date1", date1);
		model.put("date2", date2);
		return "query/JobTimeExecution";
	}

	/**
	 * form a query page for a job. The servlet should no be visible to users at
	 * all.
	 * 
	 * @param model
	 *            MVC model
	 * @return link to the JSP query page
	 */
	@RequestMapping(value = "/job/query", method = RequestMethod.GET)
	public String initFindForm(Map<String, Object> model) {
		model.put("username", getPrincipalName());
		CassandraRequester cr = new CassandraRequester();
		model.put("value", cr.getExample("jobid"));
		return "query/JobLog";
	}

	/**
	 * form a report page for a job execution time statistics.
	 * 
	 * @param date1
	 *            initial date for requested statistics
	 * @param date2
	 *            final date for requested statistics
	 * @param alldates
	 *            All available jobs are analyzed if alldates="AllDates,off"
	 * @param model
	 *            MVC model
	 * @return link to the JSP query page
	 */
	@RequestMapping(value = "/stat/exectime/results", method = RequestMethod.GET)
	public String findExecTimeData(@RequestParam("date1") String date1, @RequestParam("date2") String date2,
			@RequestParam(value = "option", required = false) String alldates, Map<String, Object> model) {
		model.put("username", getPrincipalName());
		final long startTime = System.currentTimeMillis();
		Calendar loccal = Calendar.getInstance();
		ExecutionTimeReader reader = new ExecutionTimeReader();
		if (alldates.equals("AllDates,off")) {
			date1 = theEaerlistDate;
			date2 = theCurrentDate;
		}

		// dates in string format
		String trimmeddate1 = date1.replaceAll("\\s", "");
		String trimmeddate2 = date2.replaceAll("\\s", "");
		// dates in long format
		long longDate1 = DateFormatter.DateParsing(date1, formaterYYMMDD);
		long longDate2 = DateFormatter.DateParsing(date2, formaterYYMMDD);
		String error = DateChecking(trimmeddate1, trimmeddate2, longDate1, longDate2);
		if (error != null) {
			model.put("error", error);
			model.put("date1", date1);
			model.put("date2", date2);
			return "query/JobTimeExecution";
		}

		if (longDate1 < CassandraReader.earliestDate())
			longDate1 = CassandraReader.earliestDate();
		if (longDate2 > loccal.getTimeInMillis())
			longDate2 = loccal.getTimeInMillis();

		date1 = DateFormatter.DateLongToString(longDate1, formaterYYMMDD);
		date2 = DateFormatter.DateLongToString(longDate2, formaterYYMMDD);
		model.put("date1", date1);
		model.put("date2", date2);
		model.put("option", alldates);
		ExecutionTimeBean res = reader.query(longDate1, longDate2);
		;
		model.put("result", res);
		Map<String, TotalExecutionTime> results = res.getDateTotal();
		StringBuilder csvline = new StringBuilder("");
		if (0 < res.getDateTotal().size()) {
			csvline.append("\'Date\',\'Total\',\'0-30 sec\',\'30-60 sec\',\'1-2 min\',\'2-10 min\',\'more 10 min\'%0A");
			for (Map.Entry<String, TotalExecutionTime> entry : results.entrySet()) {
				csvline.append("\'" + entry.getKey() + "\',\'" + entry.getValue().getTotal() + "\',\'" + entry.getValue().getTotal0_30s()
						+ "\',\'" + entry.getValue().getTotal30_60s() + "\',\'" + entry.getValue().getTotal1_2m() + "\',\'"
						+ entry.getValue().getTotal2_10m() + "\',\'" + entry.getValue().getTotal10m() + "\'%0A");
			}
		}
		model.put("csvfile", csvline.toString());
		model.put("ndays", res.getDateTotal().size() - 1);
		final long endTime = System.currentTimeMillis();
		model.put("timeExecution", (endTime - startTime));
		return "/reports/TimeExecution";
	}

	/**
	 * form result page for one job with a given job ID.
	 * 
	 * @param jobid
	 *            job ID
	 * @param model
	 *            MVC model
	 * 
	 * @return link to the JSP query page
	 * @throws IOException
	 */
	@RequestMapping(value = "/job/results", method = RequestMethod.GET)
	public String findJob(@RequestParam("IdJob") String jobid, Map<String, Object> model) throws IOException {
		model.put("username", getPrincipalName());
		final long startTime = System.currentTimeMillis();
		JobReader reader = new JobReader();
		model.put("result", reader.readJobLog(jobid));
		final long endTime = System.currentTimeMillis();
		model.put("timeExecution", (endTime - startTime));
		model.put("IdJob", jobid);

		// prepare archive file for the job for downloading
		ArchivedJob aj = new ArchivedJob(jobid);
		try {
			model.put("jobarchive", aj.prepareJobArchiveToWeb());
		} catch (IOException e) {
			log.error("JobController.prepareJobArchiveToWeb: IO exception with job archive file");
			log.error(e.getLocalizedMessage(), e.getCause());
		}
		// add a direct link to the job
		String remotelink = "http://www.compbio.dundee.ac.uk/www-jpred/results/" + jobid + "/" + jobid + ".results.html";
		URL remotelinkurl = new URL(remotelink);
		HttpURLConnection httpConnection_remotelinkurl = (HttpURLConnection) remotelinkurl.openConnection();
		if (199 < httpConnection_remotelinkurl.getResponseCode() && httpConnection_remotelinkurl.getResponseCode() < 300) {
			model.put("jobremotelink", remotelink);
		}
		return "reports/Job";
	}

}
